<?xml version="1.0"  encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects. 
======================================================================= -->
<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan">


<xsl:template name="standard-analysis-methods">
    <xsl:param name="reports-element"/>
    <xsl:param name="packages-element"/>
    <xsl:param name="subsystem-name"/>
    <xsl:param name="single-package"/>
    <xsl:param name="package-element"/>

    <html>
	<head>
         <link rel="stylesheet" type="text/css" href="../stylesheet.css"/>
	</head>
    <body>

   <xsl:call-template name="page-header">
      <xsl:with-param name="page-name">
       <xsl:choose>
           <xsl:when test="$single-package='true'"><xsl:value-of select="concat('Method Analysis - ',' ',/radar-report/@system,' - ', $package-element/@name)"/></xsl:when>
            <xsl:when test="not($subsystem-name)"><xsl:value-of select="concat('Method Analysis - ',/radar-report/@system)"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="concat('Method Analysis - ',' ',/radar-report/@system,' ',$subsystem-name)"/></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
      <xsl:with-param name="main-menu-page-name" select="'analysis'"/>
      <xsl:with-param name="page-type"  select="'method analysis'"/>
       <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
       <xsl:with-param name="single-package" select="$single-package"/>
       <xsl:with-param name="package-name">
            <xsl:choose>
                <xsl:when test="$single-package='true'"><xsl:value-of select="$package-element/@name"/></xsl:when>
                <xsl:otherwise></xsl:otherwise>
            </xsl:choose>
        </xsl:with-param>
   </xsl:call-template>

    <hr size="1"/>

    <p>
        In the analysis below, the methods are ranked by number of changes to method multiplied by the method's cyclomatic
        complexity. In many systems, this number will be an indicator on methods that should be refactored. The reason for
        this is that methods with high complexity will have a tendency to be hard to maintain.</p>

    <table class="details">
        <tr>
            <th>Rank</th><th>Changes</th><th>Source Statements</th><th>Cyclomatic Complexity</th>
            <xsl:if test="((not($single-package = 'true')) and ($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/test-metrics)) or
							(($single-package = 'true') and ($package-element/statistics/test-metrics))">
                <th>Test Coverage</th>
            </xsl:if>
            <th>Package</th><th>Class</th><th>Method</th>
        </tr>

        <xsl:choose>
        <xsl:when test="$single-package='true'">
            <xsl:for-each select="$package-element/classes/class/methods/method[statistics/ncss/value[position()=last()]/@value != '']">
                <xsl:sort select="count(statistics/ncss/value[@changed-since-last='true']) * statistics/ccn/value[position()=last()]/@value" order="descending" data-type="number"/>
                <xsl:if test="position() &lt;= 100">
                    <xsl:apply-templates select="current()">
						<xsl:with-param name="packages-element" select="$packages-element" />
						<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
						<xsl:with-param name="single-package" select="$single-package"/>
						<xsl:with-param name="package-element" select="$package-element"/>
                        <xsl:with-param name="position" select="position()"/>
					</xsl:apply-templates>
                </xsl:if>
            </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
            <xsl:for-each select="$packages-element/package[starts-with(@subsystem,$subsystem-name)]/classes/class/methods/method[statistics/ncss/value[position()=last()]/@value != '']">
                <xsl:sort select="count(statistics/ncss/value[@changed-since-last='true']) * statistics/ccn/value/@value" order="descending" data-type="number"/>
                <xsl:if test="position() &lt;= 100">
                    <xsl:apply-templates select="current()">
						<xsl:with-param name="packages-element" select="$packages-element" />
						<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
						<xsl:with-param name="single-package" select="$single-package"/>
						<xsl:with-param name="package-element" select="$package-element"/>
                        <xsl:with-param name="position" select="position()"/>
					</xsl:apply-templates>
                </xsl:if>
            </xsl:for-each>
        </xsl:otherwise>
        </xsl:choose>
    </table>

    </body>
    </html>

</xsl:template>

<xsl:template name="standard-analysis-changed-since-last-methods">
    <xsl:param name="reports-element"/>
    <xsl:param name="packages-element"/>
    <xsl:param name="subsystem-name"/>
    <xsl:param name="single-package"/>
    <xsl:param name="package-element"/>

    <html>
	<head>
         <link rel="stylesheet" type="text/css" href="../stylesheet.css"/>
	</head>
    <body>

   <xsl:call-template name="page-header">
      <xsl:with-param name="page-name">
       <xsl:choose>
           <xsl:when test="$single-package='true'"><xsl:value-of select="concat('Methods Changed Since Last Version Analysis - ',' ',/radar-report/@system,' - ', $package-element/@name)"/></xsl:when>
            <xsl:when test="not($subsystem-name)"><xsl:value-of select="concat('Methods Changed Since Last Version Analysis - ',/radar-report/@system)"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="concat('Methods Changed Since Last Version Analysis - ',' ',/radar-report/@system,' ',$subsystem-name)"/></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
       <xsl:with-param name="main-menu-page-name" select="'analysis'"/>  
      <xsl:with-param name="page-type"  select="'methods changed since last version analysis'"/>
       <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
       <xsl:with-param name="single-package" select="$single-package"/>
       <xsl:with-param name="package-name">
            <xsl:choose>
                <xsl:when test="$single-package='true'"><xsl:value-of select="$package-element/@name"/></xsl:when>
                <xsl:otherwise></xsl:otherwise>
            </xsl:choose>
        </xsl:with-param>
   </xsl:call-template>

    <hr size="1"/>

    <p>
        In the analysis below, the methods are ranked the lack of test coverage of the methods multiplied by their complexity.
        This analysis shows the methods that have been changed since the last build. Use this report to identify complex methods that
        are that are not being tested when added or changed.</p>
    <table class="details">
        <tr>
            <th>Rank</th><th>Changes</th><th>Source Statements</th><th>Cyclomatic Complexity</th>
            <xsl:if test="((not($single-package = 'true')) and ($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/test-metrics)) or
							(($single-package = 'true') and ($package-element/statistics/test-metrics))">
            <th>Test Coverage</th>
        </xsl:if>
            <th>Package</th><th>Class</th><th>Method</th>
        </tr>

        <xsl:choose>
        <xsl:when test="$single-package='true'">
            <xsl:for-each select="$package-element/classes/class/methods/method[(statistics/ncss/value[position()=last()]/@value != '') and (statistics/ncss/value[position()=last()]/@changed-since-last='true')]">
                <xsl:sort select="(1.0 - statistics/line-coverage/value[position()=last()]/@value) * statistics/ccn/value[position()=last()]/@value" order="descending" data-type="number"/>
                <xsl:apply-templates select="current()">
                    <xsl:with-param name="packages-element" select="$packages-element" />
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="single-package" select="$single-package"/>
                    <xsl:with-param name="package-element" select="$package-element"/>
                    <xsl:with-param name="position" select="position()"/>
                </xsl:apply-templates>
            </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
            <xsl:for-each select="$packages-element/package[starts-with(@subsystem,$subsystem-name)]/classes/class/methods/method[(statistics/ncss/value[position()=last()]/@value != '') and (statistics/ncss/value[position()=last()]/@changed-since-last='true')]">
                <xsl:sort select="(1 - statistics/line-coverage/value[position()=last()]/@value) * statistics/ccn/value[position()=last()]/@value" order="descending" data-type="number"/>
                <xsl:apply-templates select="current()">
                    <xsl:with-param name="packages-element" select="$packages-element" />
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="single-package" select="$single-package"/>
                    <xsl:with-param name="package-element" select="$package-element"/>
                    <xsl:with-param name="position" select="position()"/>
                </xsl:apply-templates>
            </xsl:for-each>
        </xsl:otherwise>
        </xsl:choose>
    </table>

    </body>
    </html>

</xsl:template>


<xsl:template match="method">
	<xsl:param name="packages-element"/>
	<xsl:param name="subsystem-name"/>
	<xsl:param name="single-package"/>
	<xsl:param name="package-element"/>
	<xsl:param name="position"/>
	 <tr>
        <td><xsl:value-of select="$position"/></td>
        <td><xsl:value-of select="count(statistics/ncss/value[@changed-since-last='true'])"/></td>
        <td><xsl:value-of select="statistics/ncss/value[position()=last()]/@value"/></td>
        <td><xsl:value-of select="statistics/ccn/value[position()=last()]/@value"/></td>
        <xsl:if test="((not($single-package = 'true')) and ($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/test-metrics)) or
							(($single-package = 'true') and ($package-element/statistics/test-metrics))">
            <td><xsl:value-of select="format-number(statistics/line-coverage/value[position()=last()]/@value,'#.##%')"/></td>
        </xsl:if>
        <td><xsl:value-of select="../../../../@name"/></td>
        <td><xsl:value-of select="../../@name"/></td>
        <td><xsl:value-of select="@name"/></td>
    </tr>
	
</xsl:template>

</xsl:stylesheet>
