<?xml version="1.0"  encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
  	extension-element-prefixes="redirect">


<xsl:template name="standard-analysis-size">
	<xsl:param name="reports-element"/>
	<xsl:param name="packages-element"/>
	<xsl:param name="subsystem-name"/>
	<xsl:param name="single-package"/>
	<xsl:param name="package-element"/>

   	<xsl:variable name="file-identifier">
   		<xsl:choose>
 			<xsl:when test="$single-package='true'">
		 		<xsl:value-of select="$package-element/@name"/>
			</xsl:when>
	        <xsl:otherwise>
	            <xsl:value-of select="$subsystem-name"/>
	        </xsl:otherwise>
        </xsl:choose>
   </xsl:variable>

   <xsl:variable name="catalog-identifier">
   		<xsl:choose>
 			<xsl:when test="$single-package='true'">
		 		<xsl:value-of select="'packages'"/>
			</xsl:when>
	        <xsl:otherwise>
	            <xsl:value-of select="'subsystems'"/>
	        </xsl:otherwise>
        </xsl:choose>
   </xsl:variable>

	<h3>Size Metrics :</h3>
	<p>
	The source statements is an indicator of the size of the system. Package volatility is a percentage measuring the fraction of packages that have been changed
	since previous release.	Method volatility is a percentage measuring the fraction of methods that have been changed
	since previous release.	Remember that this metric might be rather high since a method rename or move will imply 2 changes - one method is removed, and one method is added.
	</p>
	<table class="details">
	 	<tr><td style="background-color: white;">
		 		<div align="center">
	              <embed width="460" height="275" src="../img/size-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	            </div>
            </td>
            <td style="background-color: white;"/>
            <td style="background-color: white;">

	            <div align="center">
	              <embed width="460" height="275" src="../img/package-volatility-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	            </div>
            </td>
	 	</tr>
	 	<tr><td style="background-color: white;">
		 		<div align="center">
	              <embed width="460" height="275" src="../img/class-volatility-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	            </div>
            </td>
            <td style="background-color: white;"/>
            <td style="background-color: white;">

	            <div align="center">
	              <embed width="460" height="275" src="../img/method-volatility-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	            </div>
            </td>
	 	</tr>
	 </table>

	<a href="javascript:winpopanalysisdetails('size-details-popup-{$file-identifier}.html')">[Size Metrics Details]</a><br/>

	<redirect:write file="{$docs-home}/{$catalog-identifier}/size-details-popup-{$file-identifier}.html">
	 <xsl:call-template name="standard-analysis-size-details">
		<xsl:with-param name="reports-element" select="$reports-element"/>
		<xsl:with-param name="packages-element" select="$packages-element"/>
		<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
		<xsl:with-param name="single-package" select="$single-package"/>
		<xsl:with-param name="package-element" select="$package-element"/>
	 </xsl:call-template>
	</redirect:write>

	<redirect:write file="{$docs-home}/img/size-image-{$file-identifier}.svg">
        <xsl:call-template name="standard-dynamics-line-graph">
            <xsl:with-param name="text-y-axis" select="'Source Statements'"/>
            <xsl:with-param name="primary-metric-group" select="'source-metrics'"/>
            <xsl:with-param name="primary-metric" select="'ncss'"/>

			<xsl:with-param name="reports-element" select="$reports-element"/>
			<xsl:with-param name="packages-element" select="$packages-element"/>
			<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
			<xsl:with-param name="single-package" select="$single-package"/>
			<xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>
	</redirect:write>

	<redirect:write file="{$docs-home}/img/package-volatility-image-{$file-identifier}.svg">
		<xsl:call-template name="standard-dynamics-line-graph">
            <xsl:with-param name="text-y-axis" select="'Package Volatility [%]'"/>
            <xsl:with-param name="primary-metric-group" select="'source-metrics'"/>
            <xsl:with-param name="primary-metric" select="'package-changed'"/>
            <xsl:with-param name="package-count-divisor" select="'true'"/>

            <xsl:with-param name="multiplier" select="'100'"/>
            <xsl:with-param name="y-axis-justification" select="'none'"/>

			<xsl:with-param name="reports-element" select="$reports-element"/>
			<xsl:with-param name="packages-element" select="$packages-element"/>
			<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
			<xsl:with-param name="single-package" select="$single-package"/>
			<xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>

	</redirect:write>

  	<redirect:write file="{$docs-home}/img/class-volatility-image-{$file-identifier}.svg">

          <xsl:call-template name="standard-dynamics-line-graph">
            <xsl:with-param name="text-y-axis" select="'Class Volatility [%]'"/>
            <xsl:with-param name="primary-metric-group" select="'source-metrics'"/>
            <xsl:with-param name="primary-metric" select="'classes-changed'"/>
            <xsl:with-param name="divisor-metric-group" select="'source-metrics'"/>
            <xsl:with-param name="divisor-metric" select="'total-classes'"/>
            <xsl:with-param name="multiplier" select="'100'"/>
            <xsl:with-param name="y-axis-justification" select="'none'"/>

			<xsl:with-param name="reports-element" select="$reports-element"/>
			<xsl:with-param name="packages-element" select="$packages-element"/>
			<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
			<xsl:with-param name="single-package" select="$single-package"/>
			<xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>
	</redirect:write>

  	<redirect:write file="{$docs-home}/img/method-volatility-image-{$file-identifier}.svg">
          <xsl:call-template name="standard-dynamics-line-graph">
            <xsl:with-param name="text-y-axis" select="'Method Volatility [%]'"/>
            <xsl:with-param name="primary-metric-group" select="'source-metrics'"/>
            <xsl:with-param name="primary-metric" select="'methods-changed'"/>
            <xsl:with-param name="divisor-metric-group" select="'source-metrics'"/>
            <xsl:with-param name="divisor-metric" select="'functions'"/>
            <xsl:with-param name="multiplier" select="'100'"/>
            <xsl:with-param name="y-axis-justification" select="'none'"/>

			<xsl:with-param name="reports-element" select="$reports-element"/>
			<xsl:with-param name="packages-element" select="$packages-element"/>
			<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
			<xsl:with-param name="single-package" select="$single-package"/>
			<xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>

	</redirect:write>

</xsl:template>

<xsl:template name="standard-analysis-size-details">
	<xsl:param name="reports-element"/>
	<xsl:param name="packages-element"/>
	<xsl:param name="subsystem-name"/>
	<xsl:param name="single-package"/>
	<xsl:param name="package-element"/>
	<html>
	<head>
         <link rel="stylesheet" type="text/css" href="../stylesheet.css"/>
	</head>
    <body>

	<table class="details">
		<tr><th>Version</th><th>Date</th><th>Packages</th><th>Classes</th><th>Abstract Cl.</th><th>Concrete Cl.</th><th>Methods</th><th>Source Stmts. [SS]</th><th>[SS]/Classes</th><th>[SS]/Methods</th><th>Volatility</th></tr>
		<xsl:for-each select="$reports-element/report">
			<xsl:sort select="@seq-id"/>
			<xsl:variable name="current-seq-id" select="@seq-id"/>

			<xsl:choose>
				<xsl:when test="$single-package='true'">
			 		<xsl:if test="$package-element/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]">
				 		<xsl:variable name="classes" select="$package-element/statistics/source-metrics/total-classes/value[@seq-id=$current-seq-id]/@value"/>
						<xsl:variable name="methods" select="$package-element/statistics/source-metrics/functions/value[@seq-id=$current-seq-id]/@value"/>
						<xsl:variable name="ncss" select="$package-element/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value"/>
						<xsl:variable name="packages" select="count($package-element/package-availability/report[@seq-id=$current-seq-id])"/>
						<xsl:variable name="packages-changed"
							select="count($package-element/package-availability/report[
								(@seq-id=$current-seq-id) and
									(../../statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value != ../../statistics/source-metrics/ncss/value[@seq-id=($current-seq-id - 1)]/@value)
								])"/>
				 		<tr>
							<td><xsl:value-of select="version"/></td>
							<td><xsl:value-of select="date"/></td>
							<td><xsl:value-of select="$packages"/></td>
							<td><xsl:value-of select="$classes"/></td>
							<td><xsl:value-of select="$package-element/statistics/source-metrics/abstract-classes/value[@seq-id=$current-seq-id]/@value"/></td>
							<td><xsl:value-of select="$package-element/statistics/source-metrics/concrete-classes/value[@seq-id=$current-seq-id]/@value"/></td>
							<td><xsl:value-of select="$methods"/></td>
							<td><xsl:value-of select="$ncss"/></td>
							<td><xsl:value-of select="format-number($ncss div $classes, '#.##')"/></td>
							<td><xsl:value-of select="format-number($ncss div $methods, '#.##')"/></td>
							<td><xsl:value-of select="format-number(100 * $packages-changed div $packages, '#.##')"/>%</td>
						</tr>
					</xsl:if>
				</xsl:when>
		        <xsl:otherwise>
				    <xsl:if test="$packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]">
					    <xsl:variable name="classes" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/total-classes/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="methods" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/functions/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="ncss" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="packages" select="count($packages-element/package[starts-with(@subsystem,$subsystem-name) and (statistics/source-metrics)]/package-availability/report[@seq-id=$current-seq-id])"/>
						<xsl:variable name="packages-changed"
							select="count($packages-element/package[starts-with(@subsystem,$subsystem-name) and (statistics/source-metrics)]/package-availability/report[
								(@seq-id=$current-seq-id) and
									(../../statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value != ../../statistics/source-metrics/ncss/value[@seq-id=($current-seq-id - 1)]/@value)

								])"/>
						<tr>
							<td><xsl:value-of select="version"/></td>
							<td><xsl:value-of select="date"/></td>
							<td><xsl:value-of select="$packages"/></td>
							<td><xsl:value-of select="$classes"/></td>
							<td><xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/abstract-classes/value[@seq-id=$current-seq-id]/@value)"/></td>
							<td><xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/concrete-classes/value[@seq-id=$current-seq-id]/@value)"/></td>
							<td><xsl:value-of select="$methods"/></td>
							<td><xsl:value-of select="$ncss"/></td>
							<td><xsl:value-of select="format-number($ncss div $classes, '#.##')"/></td>
							<td><xsl:value-of select="format-number($ncss div $methods, '#.##')"/></td>
							<td><xsl:value-of select="format-number(100 * $packages-changed div $packages, '#.##')"/>%</td>
						</tr>
					</xsl:if>
		        </xsl:otherwise>
		    </xsl:choose>

		</xsl:for-each>
	</table>

	</body>
	</html>

</xsl:template>



</xsl:stylesheet>
