<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects. 
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink">

<xsl:variable name="canvas-color" select="'#BFDE1D'"/>
<xsl:variable name="axis-color" select="'black'"/>
<xsl:variable name="axis-label-color" select="'#FE9F34'"/>

<xsl:template name="common-graph">
    <xsl:param name="text-x-axis"/>	
    <xsl:param name="text-y-axis"/>    
  	<xsl:param name="width"/>
  	<xsl:param name="height"/>
  	
	<defs>
		<marker id="Triangle"
		  viewBox="0 0 10 10" refX="0" refY="5" 
		  markerUnits="strokeWidth"
		  markerWidth="4" markerHeight="3"
		  orient="auto">
		  <path d="M 0 0 L 10 5 L 0 10 z" />
		</marker>
	</defs> 
  
  <!-- Show outline of canvas using 'rect' element -->
  <rect x="1" y="1" fill="none" stroke="{$canvas-color}" stroke-width="1">
  	<xsl:attribute name="width"><xsl:value-of select="$width - 2"/></xsl:attribute>
  	<xsl:attribute name="height"><xsl:value-of select="$height - 2"/></xsl:attribute>
  </rect>              
  
  <!-- set up axis-->      
  <g stroke="{$axis-color}" >
    <!-- Vertical Axis -->
    <line stroke-width="1" marker-end="url(#Triangle)">
    	<xsl:attribute name="x1"><xsl:value-of select="ceiling($width * 0.10)"/></xsl:attribute>
    	<xsl:attribute name="x2"><xsl:value-of select="ceiling($width * 0.10)"/></xsl:attribute>
    	<xsl:attribute name="y1"><xsl:value-of select="ceiling($height * 0.90)"/></xsl:attribute>
    	<xsl:attribute name="y2"><xsl:value-of select="ceiling($height * 0.10)"/></xsl:attribute>
    </line>                    
    <!-- Horisontal Axis -->
    <line stroke-width="1" marker-end="url(#Triangle)">
    	<xsl:attribute name="x1"><xsl:value-of select="ceiling($width * 0.10)"/></xsl:attribute>
    	<xsl:attribute name="x2"><xsl:value-of select="ceiling($width * 0.90)"/></xsl:attribute>
    	<xsl:attribute name="y1"><xsl:value-of select="ceiling($height * 0.90)"/></xsl:attribute>
    	<xsl:attribute name="y2"><xsl:value-of select="ceiling($height * 0.90)"/></xsl:attribute>
    </line>
  </g>
    
  <!-- set axis text -->
  <text font-family="Verdana" fill="{$axis-label-color}">
        <xsl:attribute name="x"><xsl:value-of select="ceiling($width * 0.95) - ceiling($width * 0.015) * string-length($text-x-axis)"/></xsl:attribute>
        <xsl:attribute name="y"><xsl:value-of select="ceiling($height * 0.98)"/></xsl:attribute>
        <xsl:attribute name="font-size"><xsl:value-of select="ceiling($width * 0.025)"/></xsl:attribute>
    	<xsl:value-of select="$text-x-axis"/>
  </text>
  
  <text font-family="Verdana" fill="{$axis-label-color}" >
        <xsl:attribute name="x"><xsl:value-of select="ceiling($width * 0.02)"/></xsl:attribute>
        <xsl:attribute name="y"><xsl:value-of select="ceiling($height * 0.06)"/></xsl:attribute>
        <xsl:attribute name="font-size"><xsl:value-of select="ceiling($width * 0.025)"/></xsl:attribute>
    	<xsl:value-of select="$text-y-axis"/>
  </text>


</xsl:template>


<xsl:template name="draw-link-box">
    <xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="text-line1"/>
	<xsl:param name="text-line2"/>
    <rect
        x="{$width * 0.79}"
        y="{$height * (0.02)}"
        fill="none"
        stroke="{$canvas-color}"
        stroke-width="1">
        <xsl:attribute name="width"><xsl:value-of select="ceiling($width * 0.13)"/></xsl:attribute>
        <xsl:attribute name="height"><xsl:value-of select="ceiling($height * 0.08)"/></xsl:attribute>
    </rect>
    <text font-family="Verdana"
        fill="{$axis-label-color}"
        x="{$width * 0.8}"
        y="{$height * (0.05)}"
        font-size="{ceiling($width * 0.015)}">
        <xsl:value-of select="$text-line1"/>
    </text>
    <text font-family="Verdana"
        fill="{$axis-label-color}"
        x="{$width * 0.8}"
        y="{$height * (0.08)}"
        font-size="{ceiling($width * 0.015)}">
        <xsl:value-of select="$text-line2"/>
    </text>
</xsl:template>    


</xsl:stylesheet>
