<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink">

<xsl:include href="xradar://stylesheets/radar/graph/common-graph.xsl"/>
<xsl:include href="xradar://stylesheets/radar/graph/common-scripts.xsl"/>

<!--
Template to style a histogram graph as specified by the input parameters.
The point-nodeset specifies the input values in the standard way as specified in the XRadar documentation.
Author: Kristoffer Kvam
Since: XRadar 0.7
-->
<xsl:template name="simple-histogram-graph">
	<xsl:param name="text-x-axis"/>
	<xsl:param name="text-y-axis"/>
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="point-nodeset"/>
	<xsl:param name="sort-descending"/>
	<xsl:param name="log"/>


	<xsl:variable name="pointset" select="xalan:nodeset($point-nodeset)"/>
	<xsl:variable name="number-of-points" select="count($pointset/points/point)"/>
	<xsl:choose>
		<xsl:when test="$log = 'true'">
	        	<xsl:message>
*****************************
x-label:<xsl:value-of select="$pointset/points/point/x-label"/>
number-of-points:<xsl:value-of select="$number-of-points"/>
*****************************
			</xsl:message>
		</xsl:when>
	</xsl:choose>

  	<svg viewBox="0 0 {$width} {$height}"
  		xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
		<xsl:call-template name="common-scripts"/>

		<xsl:call-template name="common-graph">
		  	<xsl:with-param name="text-x-axis" select="$text-x-axis"/>
		  	<xsl:with-param name="text-y-axis" select="$text-y-axis"/>
		  	<xsl:with-param name="width" select="$width"/>
			<xsl:with-param name="height" select="$height"/>
		</xsl:call-template>

		<xsl:variable name="max-y">
			<xsl:for-each select="$pointset/points/point">
				<xsl:sort select="y-coordinate" 
					  order="descending" 
					  data-type="number"/>
				<xsl:if test="position() = 1">
					<xsl:value-of select="number(y-coordinate)"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>

		<xsl:for-each select="$pointset/points/point">
			<xsl:sort
				select="y-coordinate"
				data-type="number"
				order="descending"/>
			<xsl:call-template name="histo-box">
				<xsl:with-param name="width" 		select="$width"/>
				<xsl:with-param name="height" 		select="$height"/>
				<xsl:with-param name="y-value" 		select="y-coordinate"/>
				<xsl:with-param name="x-value" 		select="x-label"/>
				<xsl:with-param name="number-of-blocks" select="$number-of-points"/>
				<xsl:with-param name="max-y-value" 	select="$max-y"/>
				<xsl:with-param name="block-number" 	select="position()"/>
				<xsl:with-param name="link" 		select="link"/>
				<xsl:with-param name="bars-fill-color" 	select="'green'"/>
				<xsl:with-param name="log"		select="$log"/>
			</xsl:call-template>
		</xsl:for-each>

        <xsl:for-each select="$pointset/points/point">
			<xsl:sort
				select="y-coordinate"
				data-type="number"
				order="descending"/>
			<xsl:call-template name="histo-box-info">
				<xsl:with-param name="width" select="$width"/>
				<xsl:with-param name="height" select="$height"/>
				<xsl:with-param name="y-value" select="y-coordinate"/>
				<xsl:with-param name="x-value" select="x-label"/>
				<xsl:with-param name="number-of-blocks" select="$number-of-points"/>
				<xsl:with-param name="max-y-value" select="$max-y"/>
				<xsl:with-param name="block-number" select="position()"/>
                <xsl:with-param name="link" select="link"/>
			</xsl:call-template>
		</xsl:for-each>
	</svg>
</xsl:template>





<xsl:template name="histo-box">
	<xsl:param name="height"/>
	<xsl:param name="width"/>
	<xsl:param name="y-value"/>
	<xsl:param name="x-value"/>
	<xsl:param name="number-of-blocks"/>
	<xsl:param name="max-y-value"/>
	<xsl:param name="block-number"/>
	<xsl:param name="link"/>
	<xsl:param name="bars-fill-color"/>
	<xsl:param name="axis-color"/>
	<xsl:param name="log"/>

	<xsl:choose>
		<xsl:when test="$log = 'true'">
			<xsl:message>
*****histo-box called with:****
height:<xsl:value-of select="$height"/>
width:<xsl:value-of select="$width"/>
y-value:<xsl:value-of select="$y-value"/>
x-value:<xsl:value-of select="$x-value"/>
number-of-blocks:<xsl:value-of select="$number-of-blocks"/>
max-y-value:<xsl:value-of select="$max-y-value"/>
block-number:<xsl:value-of select="$block-number"/>
link:<xsl:value-of select="$link"/>
bars-fill-color:<xsl:value-of select="$bars-fill-color"/>
axis-color:<xsl:value-of select="$axis-color"/>
log:<xsl:value-of select="$log"/>
*******************************
			</xsl:message>
		</xsl:when>	
	</xsl:choose>

	<xsl:variable name="H" select="$height * 0.80 * $y-value div $max-y-value"/>
	<xsl:variable name="W" select="$width * 0.80 div $number-of-blocks"/>

	<a xlink:href="{$link}">
		<rect fill="blue" filter="url(#shadow_filter)">
	    	<xsl:attribute name="width">
	    		<xsl:value-of select="$W - 10"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="x">
	    		<xsl:value-of select="$width * 0.10 + $W * ($block-number - 1)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
	    		<xsl:value-of select="$height*0.90 - $H"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="height">
	    		<xsl:value-of select="$H"/>
	    	</xsl:attribute>
	    </rect>
		<rect
	    	stroke="black"
	    	stroke-width="2"
	    	onmouseover="onEvent(evt, '{$x-value}', 'visible');" onmouseout="onEvent(evt, '{$x-value}', 'hidden');">
			<xsl:attribute name="fill"><xsl:value-of select="$bars-fill-color"/></xsl:attribute>
	    	<xsl:attribute name="width">
	    		<xsl:value-of select="$W - 10"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="x">
	    		<xsl:value-of select="$width * 0.10 + $W * ($block-number - 1)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
	    		<xsl:value-of select="$height*0.90 - $H"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="height">
	    		<xsl:value-of select="$H"/>
	    	</xsl:attribute>
	    </rect>
    </a>

    <xsl:if test="$y-value &gt; 0">
	    <text font-family="Verdana" fill="green" >
	    	<xsl:attribute name="font-size">
			<xsl:value-of select="ceiling($width * 0.02)"/>
    	</xsl:attribute>
	    	<xsl:attribute name="x">
    			<xsl:value-of select="$width* 0.070 + $width div $number-of-blocks div 2 + $W * ($block-number - 1)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
	    		<xsl:value-of select="$height*0.88 - $H"/>
	    	</xsl:attribute>
	    	<xsl:value-of select="format-number($y-value,'#.##')"/>
		</text>
	</xsl:if>

</xsl:template>

<xsl:template name="histo-box-info">
	<xsl:param name="height"/>
	<xsl:param name="width"/>
	<xsl:param name="y-value"/>
	<xsl:param name="x-value"/>
	<xsl:param name="number-of-blocks"/>
	<xsl:param name="max-y-value"/>
	<xsl:param name="block-number"/>
    <xsl:param name="link"/>

	<xsl:variable name="H" select="$height * 0.80 * $y-value div $max-y-value"/>
	<xsl:variable name="W" select="$width * 0.80 div $number-of-blocks"/>

	<g style="visibility:hidden;">
				  <xsl:attribute name="id"><xsl:value-of select="$x-value"/></xsl:attribute>

          <a xlink:href="{$link}">

		<text font-family="Verdana" fill="black" >
	    	<xsl:attribute name="font-size">
				<xsl:value-of select="ceiling($width * 0.02)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="x">
				<xsl:value-of select="$width* 0.070 + $width div $number-of-blocks div 2 + $W * ($block-number - 1)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
		    	<xsl:value-of select="$height*0.94"/>
	    	</xsl:attribute>
	    	<xsl:value-of select="$x-value"/>
		</text>
        </a>
      </g>
</xsl:template>

</xsl:stylesheet>
