<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink">

	<xsl:import href="xradar://stylesheets/radar/graph/common-scripts.xsl"/>
	<xsl:import href="xradar://stylesheets/util/text/text.wrapping.template.xsl"/>

	<xsl:template name="subsystem-graph">
	<xsl:param name="text-x-axis"/>
	<xsl:param name="text-y-axis"/>
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="subsystems-nodeset"/>

	<xsl:variable name="subsystem-nodeset" select="xalan:nodeset($subsystems-nodeset)"/>


  	<svg viewBox="0 0 {$width} {$height}"
  		xmlns="http://www.w3.org/2000/svg" xmlns:link="http://www.w3.org/1999/xlink">

  		<xsl:call-template name="common-scripts"/>

		<xsl:variable name="number-of-levels">
			<xsl:for-each select="$subsystem-nodeset/subsystems/subsystem">
				<xsl:sort select="level" order="descending" data-type="number"/>
				<xsl:if test="position() = 1">
					<xsl:value-of select="level"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>

		<xsl:for-each select="$subsystem-nodeset/subsystems/subsystem">
			<xsl:sort select="level" order="ascending" data-type="number"/>
			<xsl:variable name="this-level" select="level"/>
            <xsl:call-template name="subsystem-box">
				<xsl:with-param name="width" select="$width"/>
				<xsl:with-param name="height" select="$height"/>
				<xsl:with-param name="subsystem-element" select="current()"/>
				<xsl:with-param name="number-of-levels" select="$number-of-levels"/>
				<xsl:with-param name="number-of-same-level-elements" select="count(../subsystem[level = $this-level])"/>
				<xsl:with-param name="same-level-element-number" select="count(preceding-sibling::subsystem[level = $this-level])+1"/>
				<xsl:with-param name="position" select="position()"/>
				<xsl:with-param name="color" select="color"/>
			</xsl:call-template>
		</xsl:for-each>

        <xsl:for-each select="$subsystem-nodeset/subsystems/subsystem">
			<xsl:sort select="level" order="ascending" data-type="number"/>
			<xsl:variable name="this-level" select="level"/>
            <g style="visibility:hidden;">
                <xsl:attribute name="id"><xsl:value-of select="concat(level,'_',count(preceding-sibling::subsystem[level = $this-level])+1)"/></xsl:attribute>
                <!-- todo : add bottom text here -->
            </g>
		</xsl:for-each>
	</svg>
</xsl:template>

<xsl:template name="subsystem-box">
	<xsl:param name="height"/>
	<xsl:param name="width"/>
	<xsl:param name="subsystem-element"/>
	<xsl:param name="number-of-levels"/>
	<xsl:param name="number-of-same-level-elements"/>
	<xsl:param name="same-level-element-number"/>
	<xsl:param name="position"/>
	<xsl:param name="color" select="'white'"/>

	<xsl:variable name="H" select="$height div ($number-of-levels)"/>
	<xsl:variable name="W" select="$width div ($number-of-same-level-elements)"/>
	<xsl:variable name="id" select="concat($subsystem-element/level,'_',$same-level-element-number)"/>

	<!--FIXME: Each module appears as 'red(ko)' and link open in the same frame...		-->
    	<xsl:variable name="link" select="$subsystem-element/link"/>
	<a>
		<xsl:attribute name="xlink:href"><xsl:value-of select="$link"/></xsl:attribute>
		<xsl:attribute name="target">_parent</xsl:attribute>
		<rect fill="grey"
			rx="10" ry="10"
			filter="url(#shadow_filter)">
    	 	   	<xsl:attribute name="width">
    				<xsl:value-of select="$W * 0.9"/>
    			</xsl:attribute>
    			<xsl:attribute name="height">
    				<xsl:value-of select="$H * 0.9"/>
    			</xsl:attribute>
        		<xsl:attribute name="x">
    				<xsl:value-of select="$W * ($same-level-element-number - 1) + 0.05 * $W"/>
    			</xsl:attribute>
    			<xsl:attribute name="y">
    				<xsl:value-of select="$H * ($subsystem-element/level - 1) + 0.05 * $H"/>
    			</xsl:attribute>
    	</rect>
    	<rect stroke="black" rx="10" ry="10" stroke-width="2" onmouseover="onEvent(evt, '{$id}', 'visible');" onmouseout="onEvent(evt, '{$id}', 'hidden');">
<!-- FIX ME: Properly externalize this color -->
    		<xsl:attribute name="fill">green</xsl:attribute>
    		<xsl:attribute name="width">
    			<xsl:value-of select="$W * 0.9"/>
    		</xsl:attribute>
    		<xsl:attribute name="height">
    			<xsl:value-of select="$H * 0.9"/>
    		</xsl:attribute>
    		<xsl:attribute name="x">
    			<xsl:value-of select="$W * ($same-level-element-number - 1) + 0.05 * $W"/>
    		</xsl:attribute>
    		<xsl:attribute name="y">
    			<xsl:value-of select="$H * ($subsystem-element/level - 1) + 0.05 * $H"/>
    		</xsl:attribute>
    	</rect>
    </a>

        <xsl:if test="not(contains($subsystem-element/name, ' '))">
        <text font-family="Verdana" fill="black" style="text-anchor: middle;">
	    	<xsl:attribute name="font-size">
    			<xsl:value-of select="ceiling($width * 0.025)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="x">
    			<xsl:value-of select="$W * ($same-level-element-number - 1) + 0.5 * $W"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
		    	<xsl:value-of select="$H * ($subsystem-element/level - 1) + 0.5 * $H"/>
	    	</xsl:attribute>
	    	<xsl:value-of select="$subsystem-element/name"/>
		</text>
        </xsl:if>

		<text font-family="Verdana" fill="black" style="text-anchor: middle;">
	    	<xsl:attribute name="font-size">
    			<xsl:value-of select="ceiling($width * 0.025)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="x">
    			<xsl:value-of select="$W * ($same-level-element-number - 1) + 0.5 * $W"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
		    	<xsl:value-of select="$H * ($subsystem-element/level - 1) + 0.5 * $H"/>
	    	</xsl:attribute>
	    	<xsl:value-of select="substring-before($subsystem-element/name, ' ')"/>
		</text>

		<text font-family="Verdana" fill="black" style="text-anchor: middle;">
	    	<xsl:attribute name="font-size">
    			<xsl:value-of select="ceiling($width * 0.025)"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="x">
    			<xsl:value-of select="$W * ($same-level-element-number - 1) + 0.5 * $W"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
		    	<xsl:value-of select="$H * ($subsystem-element/level - 1) + 0.65 * $H"/>
	    	</xsl:attribute>
	    	<xsl:value-of select="substring-after($subsystem-element/name, ' ')"/>
		</text>


</xsl:template>


</xsl:stylesheet>
