<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:redirect="http://xml.apache.org/xalan/redirect"
 extension-element-prefixes="redirect">

   <xsl:output
    method="xml"
    indent="yes"
    encoding="UTF-8"/>

   <xsl:include href="xradar://stylesheets/radar/statics/process/master-merge.xsl"/>

	<xsl:param       name="input-report-file" />
    <xsl:variable    name="ncss-report-doc" select="document($input-report-file)"/>


	<xsl:variable
    name="report-source"
    select="'JavaNCSS'" />
	<xsl:variable
    name="report-source-link"
    select="'http://www.kclee.com/clemens/java/javancss/'"/>

	<xsl:template
    match="Stats">
      <xsl:copy>
		 <xsl:apply-templates select="@*|node()"/>
		 <xsl:apply-templates
          select="$ncss-report-doc/javancss" mode="Package">
		  <xsl:with-param
             name="package">
             	<xsl:value-of select="../@name" />
            </xsl:with-param>
         </xsl:apply-templates>
      </xsl:copy>
   </xsl:template>



   <xsl:template
    match="Class">
      <xsl:copy>
         <xsl:apply-templates select="@*|node()"/>
         <xsl:apply-templates
          select="$ncss-report-doc/javancss" mode="Class">
            <xsl:with-param
             	name="source-file">
             	<xsl:value-of select="node()" />
		  	</xsl:with-param>
		  	<xsl:with-param
             	name="detail-analysed">
				<xsl:if test="node()/analysed-in-detail">
					true
				</xsl:if>
            </xsl:with-param>
         </xsl:apply-templates>
      </xsl:copy>
   </xsl:template>


   <xsl:template
    match="javancss" mode="Package">
		<xsl:param name="package" />
		<xsl:for-each select="packages/package[name = $package]">
			<xsl:copy-of select="functions"/>
			<xsl:copy-of select="ncss"/>
			<xsl:copy-of select="javadocs"/>
			<xsl:copy-of select="javadoc_lines"/>
			<xsl:copy-of select="single_comment_lines"/>
			<xsl:copy-of select="multi_comment_lines"/>
		</xsl:for-each>
   </xsl:template>

   <xsl:template
    	match="javancss" mode="Class">
		<xsl:param name="source-file" />
		<xsl:param name="detail-analysed"/>
		<xsl:variable name="source-file-normalized" select="normalize-space($source-file)"/>

		<xsl:for-each select="objects/object[name = $source-file-normalized]">
			<xsl:element name="standard-metrics">
				<xsl:copy-of select="functions"/>
				<xsl:copy-of select="ncss"/>
				<xsl:copy-of select="javadocs"/>
			</xsl:element>
			<xsl:element name="functions">
				<xsl:for-each select="../../functions/function[starts-with(name,concat($source-file-normalized,'.'))]">
					<xsl:copy>
						<xsl:apply-templates select="@*|node()"/>
					</xsl:copy>
				</xsl:for-each>
			</xsl:element>
			<xsl:if test="not(normalize-space($detail-analysed) = true)">
				<analysed-in-detail/>
			</xsl:if>
		</xsl:for-each>
   </xsl:template>


   <xsl:template name="package-addition"/>

</xsl:stylesheet>
