<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:lxslt="http://xml.apache.org/xslt"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
    extension-element-prefixes="redirect">


<xsl:template match="IntegratedAnalysis" mode="scorecard-code">

	<xsl:variable name="packages-element" select="Packages"/>

   	<html>
      <head>
         <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>

         <xsl:call-template name="page-header">
  	  	   <xsl:with-param name="main-menu-page-name" select="'scorecard'"/>
  	  	   <xsl:with-param name="page-name" select="'code'"/>
  	  	   <xsl:with-param name="dynamics-page-name" select="'code'"/>
  	  	   <xsl:with-param name="root-element" select="current()"/>
  	     </xsl:call-template>

		<h3>Code Metrics</h3>
		For each histogram column, you can roll over the diagrams to get details on the contents on which subsystem the column relates to.
        Click on it to go to a detailed view. On the bottom of the screen the data is presented in tabular form.
		<br/>
		<table width="100%" class="details">
			<tr>
				<td style="background-color: white;">
			 		<div align="top">
		              <embed width="400" height="300" src="img/subsystem-size-histogram.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
	            </td>
				<td style="background-color: white;">
			 		<div align="top">
		              <embed width="400" height="300" src="img/subsystem-complexity-per-method-histogram.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
				</td>
	        </tr>
	        <tr>
	           <td style="background-color: white;">
			 		<div align="top">
		              <embed width="400" height="300" src="img/subsystem-violations-per-ncss-histogram.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
	            </td>
	            <td style="background-color: white;">
			 		<div align="top">
		              <embed width="400" height="300" src="img/subsystem-style-errors-per-ncss-histogram.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
	            </td>
			</tr>
		</table>
		<table width="100%" class="details">


		    <xsl:call-template name="metric-header-code"/>

			<xsl:call-template name="subsystem-code-metrics">
				<xsl:with-param name="subsystem-name" select="'Total'"/>
				<xsl:with-param name="subsystem-link" select="'none'"/>
				<xsl:with-param name="source-statements-in-subsystem" select="sum($packages-element/Package/Stats/ncss)"/>
				<xsl:with-param name="methods-in-subsystem" select="sum($packages-element/Package/Stats/functions)"/>
				<xsl:with-param name="cyclomatic-complexity-in-subsystem" select="sum($packages-element/Package/Stats/ccn)"/>
				<xsl:with-param name="total-packages" select="count($packages-element/Package/Stats)"/>
				<xsl:with-param name="total-classes" select="sum($packages-element/Package/Stats/TotalClasses)"/>
				<xsl:with-param name="bc-classes" select="sum($packages-element/Package/Stats/business-critical-classes)"/>
				<xsl:with-param name="violations" select="sum($packages-element/Package/Stats/code-violations)"/>
				<xsl:with-param name="style-errors" select="sum($packages-element/Package/Stats/style-errors)"/>
                <xsl:with-param name="duplicated-tokens" select="sum($packages-element/Package/Stats/duplication-metrics/tokens)"/>
			</xsl:call-template>

			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
				<xsl:variable name="file-name"><xsl:call-template name="subsystem-url">
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="anchor" select="'code-metrics'"/>
                </xsl:call-template></xsl:variable>
				<xsl:call-template name="subsystem-code-metrics">
					<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="subsystem-link" select="$file-name"/>
					<xsl:with-param name="source-statements-in-subsystem" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/ncss)"/>
					<xsl:with-param name="methods-in-subsystem" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/functions)"/>
					<xsl:with-param name="cyclomatic-complexity-in-subsystem" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/ccn)"/>
					<xsl:with-param name="total-packages" select="count($packages-element/Package[subsystem/@name = $subsystem-name]/Stats)"/>
					<xsl:with-param name="total-classes" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/TotalClasses)"/>
					<xsl:with-param name="bc-classes" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/business-critical-classes)"/>
					<xsl:with-param name="violations" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/code-violations)"/>
					<xsl:with-param name="style-errors" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/style-errors)"/>
					<xsl:with-param name="duplicated-tokens" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/duplication-metrics/tokens)"/>
				</xsl:call-template>
			</xsl:for-each>

			<xsl:call-template name="subsystem-code-metrics">
				<xsl:with-param name="subsystem-name" select="'Not Classified'"/>
				<xsl:with-param name="subsystem-link"><xsl:call-template name="subsystem-url">
					<xsl:with-param name="subsystem-name" select="'not-classified'"/>
					<xsl:with-param name="anchor" select="'code-metrics'"/>
				</xsl:call-template></xsl:with-param>
				<xsl:with-param name="source-statements-in-subsystem" select="sum($packages-element/Package[not(subsystem)]/Stats/ncss)"/>
				<xsl:with-param name="methods-in-subsystem" select="sum($packages-element/Package[not(subsystem)]/Stats/functions)"/>
				<xsl:with-param name="cyclomatic-complexity-in-subsystem" select="sum($packages-element/Package[not(subsystem)]/Stats/ccn)"/>
				<xsl:with-param name="total-packages" select="count($packages-element/Package[not(subsystem)]/Stats)"/>
				<xsl:with-param name="total-classes" select="sum($packages-element/Package[not(subsystem)]/Stats/TotalClasses)"/>
				<xsl:with-param name="bc-classes" select="sum($packages-element/Package[not(subsystem)]/Stats/business-critical-classes)"/>
				<xsl:with-param name="violations" select="sum($packages-element/Package[not(subsystem)]/Stats/code-violations)"/>
				<xsl:with-param name="style-errors" select="sum($packages-element/Package[not(subsystem)]/Stats/style-errors)"/>
				<xsl:with-param name="duplicated-tokens" select="sum($packages-element/Package[not(subsystem)]/Stats/duplication-metrics/tokens)"/>
			</xsl:call-template>

	 	</table>

	<h3>Duplications</h3>
		<p>
            The matrix below presents in what way and to what degree code duplications are found in the
            modules in the analysed system. Each dot represents a duplication between two classes.  The size of the dots represents the number of
            duplications the two classes are involved in between each other. Dots on the
            diagonal represent duplications within a class. Notice that each side of the diagonal is equal. This
            is no coincidence since they are each others mirror image.
        </p>
		<p>
            By pointing your mouse over the dots you will see the details of that duplication. Click to
            go to the class.
        </p>
        <br/>
		<table width="100%" class="details" align="center">
			<tr>
				<td style="background-color: white;">
			 		<div align="top">
		              <embed width="800" height="800" src="img/subsystem-code-duplication-matrix.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
	            </td>
	        </tr>
	    </table>


	<a name="pareto"></a><h3>Pareto Principle</h3>
    <p>
    <b>Theory</b><br/>
    The Pareto Principle states that a minority of input produces the majority of results. This principle is often called the 80%:20% rule. The principle originates
    from economics, but is often just as applicable to other types of systems such as software. In the following graphs Aggregated Source Statements and  Aggregated
    Cyclomatic Complexity per package is plotted.<br/>
    <b>Needed Actions</b><br/>
    According to the theroy the aggregated results will in most cases go though the 80:20 / 70:30 box. That
    is not necessarily assert a bad design. Still, it proves the point that most of the logics and development effort in the system is focused on a limited number of
    packages. You may in such cases consider to split up the main packages or pull together the smallest packages to get
    the curve to go below the 80:20 / 70:30 box. That will probably make the package structure more balanced. Note that if all the packages are of equal size, the curve
    will be linear (y=x).<br/><br/>
    See the explanations for
    <a href="overview-explanations.html#EXinstability ">Source Statements</a> and
    <a href="overview-explanations.html#EXccn">Cyclomatic Complexity</a> for more on their definitions.
    </p>
    <p>
    <table border="0">
    <tr>
    	<td width="45%">
    		<div align="top">
	           <embed width="450" height="450" src="img/aggregatesourcestatements-packagenumber-graph.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	        </div>
	    </td>
    	<td width="10%"></td>
    	<td width="45%" valign="top">
    		<table class="details" width="100%">
    		<tr>
    			<th>#</th><th>Package</th><th>Source Statements</th>
    		</tr>
    		<xsl:for-each select="Packages/Package">
    			<xsl:sort select="Stats/ncss" order="descending" data-type="number"/>
					<xsl:if test="position() &lt;= 20">
				        <tr>
				        <td><xsl:value-of select="position()"/></td>
				        <td>
					        <a>
		                       <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="@name"/>
                                  </xsl:call-template></xsl:attribute>
		                       <xsl:value-of select="@name"/>
		                    </a>
                        </td>
                        <td><xsl:value-of select="Stats/ncss"/></td>
                        </tr>
				    </xsl:if>
		    </xsl:for-each>
		    </table>
    	</td>
    </tr>
    <tr>
    	<td colspan="3"><br/></td>
    </tr>
    <tr>
    	<td width="45%">
    		<div align="top">
	           <embed width="450" height="450" src="img/aggregatecomplexity-packagenumber-graph.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	        </div>
	    </td>
    	<td width="10%"></td>
    	<td width="45%" valign="top">
    		<table class="details" width="100%">
    		<tr>
    			<th>#</th><th>Package</th><th>Cyclomatic Complexity</th>
    		</tr>
    		<xsl:for-each select="Packages/Package">
    			<xsl:sort select="Stats/ccn" order="descending" data-type="number"/>
					<xsl:if test="position() &lt;= 20">
				        <tr>
				        <td><xsl:value-of select="position()"/></td>
				        <td>
					        <a>
		                       <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="@name"/>
                                  </xsl:call-template></xsl:attribute>
		                       <xsl:value-of select="@name"/>
		                    </a>
                        </td>
                        <td><xsl:value-of select="Stats/ccn"/></td>
                        </tr>
				    </xsl:if>
		    </xsl:for-each>
		    </table>
    	</td>
    </tr>
    </table>
    </p>
      </body>
   </html>


</xsl:template>

<xsl:template name="metric-header-code">
	<xsl:param name="single-package"/>

	<tr>
       <th>Code Metrics</th>
		<xsl:if test="not($single-package='true')">
			<th>Total Packages</th>
		</xsl:if>
		<th>Total Classes</th>
		<th>BC Classes</th>
		<th>Source Statements</th>
		<th>Cyclomatic Complexity</th>
		<th>Cmplx. per Stmnt.</th>
		<th>Cmplx. per Meth.</th>
		<th>Violations/ Source Stmts.</th>
		<th>Style errors/ Source Stmts.</th>
		<th>Duplicated Tokens / Source Stmts.</th>
    </tr>
</xsl:template>

<xsl:template name="subsystem-code-metrics">
	<xsl:param name="subsystem-name"/>
	<xsl:param name="subsystem-link"/>
	<xsl:param name="source-statements-in-subsystem"/>
	<xsl:param name="methods-in-subsystem"/>
	<xsl:param name="cyclomatic-complexity-in-subsystem"/>
	<xsl:param name="total-packages"/>
	<xsl:param name="total-classes"/>
	<xsl:param name="bc-classes"/>
	<xsl:param name="violations"/>
	<xsl:param name="style-errors"/>
	<xsl:param name="duplicated-tokens"/>
	<xsl:param name="single-package"/>

	<tr>
	 <td align="left">
	 	<xsl:choose>
	 		<xsl:when test="$subsystem-link != 'none'">
			    <a href="{$subsystem-link}">
			       <xsl:value-of select="$subsystem-name"/>
			    </a>
			</xsl:when>
			<xsl:otherwise>
				<b><xsl:value-of select="$subsystem-name"/></b>
			</xsl:otherwise>
	    </xsl:choose>
	 </td>
	 <xsl:if test="not($single-package='true')">
       	<td align="right"><xsl:value-of select="$total-packages"/></td>
     </xsl:if>
	 <td align="right"><xsl:value-of select="$total-classes"/></td>
	 <td align="right"><xsl:value-of select="$bc-classes"/></td>
	 <td align="right"><xsl:value-of select="$source-statements-in-subsystem"/></td>
	 <td align="right"><xsl:value-of select="$cyclomatic-complexity-in-subsystem"/></td>
	 <td align="right">
	 <xsl:choose>
	 	<xsl:when test="$source-statements-in-subsystem=0">
		 		0
	 	</xsl:when>
	 	<xsl:otherwise>
	 		<xsl:value-of select="format-number($cyclomatic-complexity-in-subsystem div $source-statements-in-subsystem, '#.##')"/>
	 	</xsl:otherwise>
	 </xsl:choose>
	 </td>
	 <td align="right">
	 <xsl:choose>
	 	<xsl:when test="$methods-in-subsystem=0">
		 		0
	 	</xsl:when>
	 	<xsl:otherwise>
	 		<xsl:value-of select="format-number($cyclomatic-complexity-in-subsystem div $methods-in-subsystem, '#.##')"/>
	 	</xsl:otherwise>
	 </xsl:choose>
	 </td>
	 <td align="right"><xsl:value-of select="format-number($violations div $source-statements-in-subsystem, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($style-errors div $source-statements-in-subsystem, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($duplicated-tokens div $source-statements-in-subsystem, '#.##')"/></td>

	</tr>
</xsl:template>



</xsl:stylesheet>
